@@ -73,6 +73,7 @@ class PillowPlugin(PluginV3):
         super().__init__(request)
 
         self._image: Image = None
+        self.images_to_write = []
 
         if request.mode.io_mode == IOMode.read:
             try:
@@ -93,6 +94,8 @@ class PillowPlugin(PluginV3):
 
             self._image = Image.open(self._request.get_file())
         else:
+            self.save_args = {}
+
             extension = self.request.extension or self.request.format_hint
             if extension is None:
                 warnings.warn(
@@ -114,6 +117,8 @@ class PillowPlugin(PluginV3):
             ) from None
 
     def close(self) -> None:
+        self._flush_writer()
+
         if self._image:
             self._image.close()
 
@@ -122,13 +127,14 @@ class PillowPlugin(PluginV3):
     def read(
         self,
         *,
-        index=None,
-        mode=None,
-        pilmode=None,
-        rotate=False,
-        exifrotate=None,
-        apply_gamma=False,
-        as_gray=None,
+        index: int = None,
+        mode: str = None,
+        rotate: bool = False,
+        apply_gamma: bool = False,
+        writeable_output: bool = True,
+        pilmode: str = None,
+        exifrotate: bool = None,
+        as_gray: bool = None,
     ) -> np.ndarray:
         """
         Parses the given URI and creates a ndarray from it.
@@ -147,16 +153,21 @@ class PillowPlugin(PluginV3):
             Convert the image to the given mode before returning it. If None,
             the mode will be left unchanged. Possible modes can be found at:
             https://pillow.readthedocs.io/en/stable/handbook/concepts.html#modes
+        rotate : bool
+            If True and the image contains an EXIF orientation tag,
+            apply the orientation before returning the ndimage.
+        apply_gamma : bool
+            If True and the image contains metadata about gamma, apply gamma
+            correction to the image.
+        writable_output : bool
+            If True, ensure that the image is writable before returning it to
+            the user. This incurs a full copy of the pixel data if the data
+            served by pillow is read-only. Consequentially, setting this flag to
+            False improves performance for some images.
         pilmode : str
             Deprecated, use `mode` instead.
-        rotate : bool
-            If set to ``True`` and the image contains an EXIF orientation tag,
-            apply the orientation before returning the ndimage.
         exifrotate : bool
             Deprecated, use `rotate` instead.
-        apply_gamma : bool
-            If ``True`` and the image contains metadata about gamma, apply gamma
-            correction to the image.
         as_gray : bool
             Deprecated. Exists to raise a constructive error message.
 
@@ -206,15 +217,27 @@ class PillowPlugin(PluginV3):
         if isinstance(index, int):
             # will raise IO error if index >= number of frames in image
             self._image.seek(index)
-            image = self._apply_transforms(self._image, mode, rotate, apply_gamma)
-            return image
+            image = self._apply_transforms(
+                self._image, mode, rotate, apply_gamma, writeable_output
+            )
         else:
-            iterator = self.iter(mode=mode, rotate=rotate, apply_gamma=apply_gamma)
+            iterator = self.iter(
+                mode=mode,
+                rotate=rotate,
+                apply_gamma=apply_gamma,
+                writeable_output=writeable_output,
+            )
             image = np.stack([im for im in iterator], axis=0)
-            return image
+
+        return image
 
     def iter(
-        self, *, mode: str = None, rotate: bool = False, apply_gamma: bool = False
+        self,
+        *,
+        mode: str = None,
+        rotate: bool = False,
+        apply_gamma: bool = False,
+        writeable_output: bool = True,
     ) -> Iterator[np.ndarray]:
         """
         Iterate over all ndimages/frames in the URI
@@ -231,12 +254,21 @@ class PillowPlugin(PluginV3):
         apply_gamma : {bool}
             If ``True`` and the image contains metadata about gamma, apply gamma
             correction to the image.
+        writable_output : bool
+            If True, ensure that the image is writable before returning it to
+            the user. This incurs a full copy of the pixel data if the data
+            served by pillow is read-only. Consequentially, setting this flag to
+            False improves performance for some images.
         """
 
         for im in ImageSequence.Iterator(self._image):
-            yield self._apply_transforms(im, mode, rotate, apply_gamma)
+            yield self._apply_transforms(
+                im, mode, rotate, apply_gamma, writeable_output
+            )
 
-    def _apply_transforms(self, image, mode, rotate, apply_gamma) -> np.ndarray:
+    def _apply_transforms(
+        self, image, mode, rotate, apply_gamma, writeable_output
+    ) -> np.ndarray:
         if mode is not None:
             image = image.convert(mode)
         elif image.mode == "P":
@@ -259,6 +291,9 @@ class PillowPlugin(PluginV3):
             image = ((image / scale) ** gamma) * scale * gain + 0.4999
             image = np.round(image).astype(np.uint8)
 
+        if writeable_output and not image.flags["WRITEABLE"]:
+            image = np.array(image)
+
         return image
 
     def write(
@@ -321,12 +356,6 @@ class PillowPlugin(PluginV3):
                 "(in ms) instead, e.g. `fps=50` == `duration=20` (1000 * 1/50)."
             )
 
-        extension = self.request.extension or self.request.format_hint
-
-        save_args = {
-            "format": format or Image.registered_extensions()[extension],
-        }
-
         if isinstance(ndimage, list):
             ndimage = np.stack(ndimage, axis=0)
             is_batch = True
@@ -354,26 +383,51 @@ class PillowPlugin(PluginV3):
         if not is_batch:
             ndimage = ndimage[None, ...]
 
-        pil_frames = list()
         for frame in ndimage:
             pil_frame = Image.fromarray(frame, mode=mode)
             if "bits" in kwargs:
                 pil_frame = pil_frame.quantize(colors=2 ** kwargs["bits"])
-            pil_frames.append(pil_frame)
-        primary_image, other_images = pil_frames[0], pil_frames[1:]
+            self.images_to_write.append(pil_frame)
 
-        if is_batch:
-            save_args["save_all"] = True
-            save_args["append_images"] = other_images
+        if (
+            format is not None
+            and "format" in self.save_args
+            and self.save_args["format"] != format
+        ):
+            old_format = self.save_args["format"]
+            warnings.warn(
+                "Changing the output format during incremental"
+                " writes is strongly discouraged."
+                f" Was `{old_format}`, is now `{format}`.",
+                UserWarning,
+            )
 
-        save_args.update(kwargs)
-        primary_image.save(self._request.get_file(), **save_args)
+        extension = self.request.extension or self.request.format_hint
+        self.save_args["format"] = format or Image.registered_extensions()[extension]
+        self.save_args.update(kwargs)
 
+        # when writing to `bytes` we flush instantly
+        result = None
         if self._request._uri_type == URI_BYTES:
+            self._flush_writer()
             file = cast(BytesIO, self._request.get_file())
-            return file.getvalue()
+            result = file.getvalue()
 
-        return None
+        return result
+
+    def _flush_writer(self):
+        if len(self.images_to_write) == 0:
+            return
+
+        primary_image = self.images_to_write.pop(0)
+
+        if len(self.images_to_write) > 0:
+            self.save_args["save_all"] = True
+            self.save_args["append_images"] = self.images_to_write
+
+        primary_image.save(self._request.get_file(), **self.save_args)
+        self.images_to_write.clear()
+        self.save_args.clear()
 
     def get_meta(self, *, index=0) -> Dict[str, Any]:
         return self.metadata(index=index, exclude_applied=False)
@@ -484,8 +538,9 @@ class PillowPlugin(PluginV3):
         height: int = self._image.height
         shape: Tuple[int, ...] = (height, width)
 
-        n_frames: int = self._image.n_frames
+        n_frames: Optional[int] = None
         if index is ...:
+            n_frames = getattr(self._image, "n_frames", 1)
             shape = (n_frames, *shape)
 
         dummy = np.asarray(Image.new(mode, (1, 1)))
@@ -496,6 +551,6 @@ class PillowPlugin(PluginV3):
         return ImageProperties(
             shape=shape,
             dtype=dummy.dtype,
-            n_images=n_frames if index is Ellipsis else None,
+            n_images=n_frames,
             is_batch=index is Ellipsis,
         )
